#include <unistd.h>
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>

/*
 *  old new
 *  1: a is newer than b 
 *  0: a and b are the same
 * -1: b is newer than a
 */

main(int argc, char *argv[]) {
	int rc;

	rc = rpmvercmp(argv[1], argv[2]);
	printf("%i\n",rc);
	exit(rc);
}

int rpmvercmp(const char * a, const char * b) {
    char oldch1, oldch2;
    char * str1, * str2;
    char * one, * two;
    int rc;
    int isnum;
    /* easy comparison to see if versions are identical */
    if (!strcmp(a, b)) return 0;
    str1 = alloca(strlen(a) + 1);
    str2 = alloca(strlen(b) + 1);
    strcpy(str1, a);
    strcpy(str2, b);
    one = str1;
    two = str2;
    /* loop through each version segment of str1 and str2 and compare them */
    while (*one && *two) {
    while (*one && !isalnum(*one)) one++;
    while (*two && !isalnum(*two)) two++;
    str1 = one;
    str2 = two;
    /* grab first completely alpha or completely numeric segment */
    /* leave one and two pointing to the start of the alpha or numeric */
    /* segment and walk str1 and str2 to end of segment */
    if (isdigit(*str1)) {
        while (*str1 && isdigit(*str1)) str1++;
        while (*str2 && isdigit(*str2)) str2++;
        isnum = 1;
    } else {
        while (*str1 && isalpha(*str1)) str1++;
        while (*str2 && isalpha(*str2)) str2++;
        isnum = 0;
    }   
    /* save character at the end of the alpha or numeric segment */
    /* so that they can be restored after the comparison */
    oldch1 = *str1;
    *str1 = '\0';
    oldch2 = *str2;
    *str2 = '\0';
    /* take care of the case where the two version segments are */
    /* different types: one numeric and one alpha */
    if (one == str1) return -1; /* arbitrary */
    if (two == str2) return -1;
    if (isnum) {
        /* this used to be done by converting the digit segments */
        /* to ints using atoi() - it's changed because long  */
        /* digit segments can overflow an int - this should fix that. */
        /* throw away any leading zeros - it's a number, right? */
        while (*one == '0') one++;
        while (*two == '0') two++;
        /* whichever number has more digits wins */
        if (strlen(one) > strlen(two)) return 1;
        if (strlen(two) > strlen(one)) return -1;
    }
    /* strcmp will return which one is greater - even if the two */
    /* segments are alpha or if they are numeric.  don't return  */
    /* if they are equal because there might be more segments to */
    /* compare */
    rc = strcmp(one, two);
    if (rc) return rc;
    /* restore character that was replaced by null above */
    *str1 = oldch1;
    one = str1;
    *str2 = oldch2;
    two = str2;
    }
    /* this catches the case where all numeric and alpha segments have */
    /* compared identically but the segment sepparating characters were */
    /* different */
    if ((!*one) && (!*two)) return 0;
    /* whichever version still has characters left over wins */
    if (!*one) return -1; else return 1;
}   
